﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Data;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Diagrams;

namespace ClassDiagram
{
    public class ClassShape : RadDiagramContainerShape
    {
        public static readonly DependencyProperty FieldsVisibilityProperty =
            DependencyProperty.Register("FieldsVisibility", typeof(Visibility), typeof(ClassShape), new PropertyMetadata(Visibility.Collapsed));

        public static readonly DependencyProperty PropertiesVisibilityProperty =
            DependencyProperty.Register("PropertiesVisibility", typeof(Visibility), typeof(ClassShape), new PropertyMetadata(Visibility.Collapsed));

        public static readonly DependencyProperty RelationshipsVisibilityProperty =
          DependencyProperty.Register("RelationshipsVisibility", typeof(Visibility), typeof(ClassShape), new PropertyMetadata(Visibility.Collapsed));

        private readonly List<PropertyContainer> propertyContainers = new List<PropertyContainer>();

        public Visibility RelationshipsVisibility
        {
            get { return (Visibility)GetValue(RelationshipsVisibilityProperty); }
            set { SetValue(RelationshipsVisibilityProperty, value); }
        }

        public Visibility FieldsVisibility
        {
            get { return (Visibility)GetValue(FieldsVisibilityProperty); }
            set { SetValue(FieldsVisibilityProperty, value); }
        }

        public Visibility PropertiesVisibility
        {
            get { return (Visibility)GetValue(PropertiesVisibilityProperty); }
            set { SetValue(PropertiesVisibilityProperty, value); }
        }

        protected override bool IsDropPossible
        {
            get
            {
                return false;
            }
        }

        public ClassShape()
            : base()
        {
            this.Loaded += OnLoaded;
            this.Connectors.RemoveAt(3);
            this.Connectors.RemoveAt(1);
        }

        public bool AddProperty()
        {
            return this.AddNewChild(1, "New Property");
        }

        public bool AddField()
        {
            return this.AddNewChild(0, "New Field");
        }

        public bool AddRelationship()
        {
            return this.AddNewChild(2, "New Relationship");
        }

        protected override void OnItemsCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            base.OnItemsCollectionChanged(sender, e);
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
            {
            }
            else if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                foreach (var item in e.NewItems.OfType<PropertyContainer>())
                {
                    item.Position = this.ContentBounds.TopLeft;
                    item.Width = this.Width;
                }
                this.RefreshConnections();
            }
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            if (this.propertyContainers.Count == 0)
            {
                var fields = new PropertyContainer() { Content = "Fields" };
                this.Bind(fields, "FieldsVisibility");
                this.propertyContainers.Add(fields);
                this.Items.Add(fields);

                var properties = new PropertyContainer() { Content = "Properties" };
                this.Bind(properties, "PropertiesVisibility");
                this.propertyContainers.Add(properties);
                this.Items.Add(properties);

                var relationships = new PropertyContainer() { Content = "Relationships" };
                this.Bind(relationships, "RelationshipsVisibility");
                this.propertyContainers.Add(relationships);
                this.Items.Add(relationships);
            }
        }

        private void Bind(PropertyContainer source, string property)
        {
            Binding visibilityBinding = new Binding(property);
            visibilityBinding.Mode = BindingMode.TwoWay;
            visibilityBinding.Source = this;

            source.SetBinding(FrameworkElement.VisibilityProperty, visibilityBinding);
        }

        private bool AddNewChild(int index, string name)
        {
            if (index >= this.propertyContainers.Count) return false;

            var child = propertyContainers[index];
            if (child == null) return false;

            if (child.Visibility == System.Windows.Visibility.Collapsed)
            {
                child.Width = this.Width;
                child.Visibility = System.Windows.Visibility.Visible;
            }

            child.Items.Add(new PropertyShape() { Content = name });
            return true;
        }

        private void UpdatePositionsAfterIndex(int start, double delta)
        {
            if (start < 0) return;

            for (int i = start + 1; i < this.propertyContainers.Count; i++)
            {
                var container = this.propertyContainers[i];
                container.Position = new Point(container.Position.X, container.Position.Y + delta);
            }
        }

        private void RefreshConnections()
        {
            if (this.Diagram == null) return;

            foreach (var connection in this.Diagram.Connections)
            {
                connection.Update();
            }
        }

        internal void UpdatePositions(PropertyContainer propertyContainer, double delta)
        {
            this.UpdatePositionsAfterIndex(this.propertyContainers.IndexOf(propertyContainer), delta);
        }

        internal void UpdateContentBounds()
        {
            var bounds = this.CalculateContentBounds(new Rect(this.Position, new Size(this.MinWidth, this.MinHeight)));
            bounds.Height += 10;
            this.ContentBounds = bounds;
        }
    }
}
